//
//  CharacterGenerator.swift
//  FruitMachine
//
//  Created by Christopher Rohl on 7/26/17.
//  Copyright © 2017 Christopher Rohl. All rights reserved.
//

import Cocoa

//The character generator ROM contains 64 8x5 glyphs.

struct Glyph {
    var pixels: [BitmapPixelsBE555.PixelData] = [BitmapPixelsBE555.PixelData]()
    
    init(inPixels: [BitmapPixelsBE555.PixelData]) {
        pixels = inPixels
    }
}

class CharacterGenerator: NSObject {
    static let CHAR_WIDTH = 5
    static let CHAR_HEIGHT = 8
    
    var ROM: [UInt8]
    var glyphs: [Glyph]
    
    init(romPath: String) {
        ROM = [UInt8](repeating: 0xCC, count: 512)
        glyphs = [Glyph](repeating: Glyph(inPixels: [BitmapPixelsBE555.PixelData]()), count: 64)
        
        super.init()
        loadROM(path: romPath)
        
        for index in 0..<64 {
            glyphs[index] = Glyph(inPixels: getCharacterPixels(charIndex: UInt8(index)))
        }
    }
    
    private func loadROM(path: String) {
        do {
            let fileContent: NSData = try NSData(contentsOfFile: path)
            fileContent.getBytes(&ROM, range: NSRange(location: 0, length: 512))
        } catch {
            print(error)
        }
    }

    private func getCharacterPixels(charIndex: UInt8) -> [BitmapPixelsBE555.PixelData] {
        var pixelArray = [UInt8](repeating: 0x00, count: CharacterGenerator.CHAR_HEIGHT)
        
        /* Instead of ignoring ASCII bit b6, we ignore bit b5. At the same time ASCII bit b6 must be inverted before it is fed to the character ROM. This way the entire character range from $40 to $7F will end up in the range $00 to $1F (twice of course). Now lower case characters are automatically translated into their corresponding upper case bit maps.
         */
        
        //Don't convert the character indexes if we're populating the glyphs array.
        for scanlineIndex in 0..<CharacterGenerator.CHAR_HEIGHT {
            pixelArray[scanlineIndex] = ROM[scanlineIndex + (Int(charIndex) * CharacterGenerator.CHAR_HEIGHT)]
        }
        
        var glyphPixels = [BitmapPixelsBE555.PixelData]()
        
        for charY in 0..<CharacterGenerator.CHAR_HEIGHT {
            for charX in 0..<8 {
                glyphPixels.append(pixelArray[Int(charY)] & (1 << charX) > 0 ? BitmapPixelsBE555.White : BitmapPixelsBE555.Black)
            }
        }
        
        return glyphPixels
    }
    
    func asciiToAppleCharIndex(ascii: UInt8) -> UInt8 {
        return (ascii & 0x1f) | (((ascii ^ 0x40) & 0x40) >> 1)
    }
    
}
